package model;

import java.util.*;

import listener.CoordinateChangeEvent;
import listener.CoordinateListener;

public class Coordinate {

	private int x = 5; // J[\̏ʒu

	private int y = 5;

	private int x0 = x;

	private int y0 = y;

	private boolean isRegion;

	private LayerList layerList;

	private List<CoordinateListener> listeners = new ArrayList<CoordinateListener>();

	public Coordinate(final LayerList list) {
		layerList = list;
	}

	public void delete() {
		cancelRegion();
		notifyToCoordinateListeners();
	}

	public void space() {
		cancelRegion();
		notifyToCoordinateListeners();
	}

	public void enter() {
		cancelRegion();
		notifyToCoordinateListeners();
	}

	public void cancelRegion() {
		isRegion = false;
		x0 = x;
		y0 = y;
	}

	public void initializeData() {
		isRegion = false;
		x = x0 = layerList.getColumnCount() / 4;
		y = y0 = layerList.getRowCount() / 4;
		notifyToCoordinateListeners();
	}

	public int getDataWidth() {
		return Math.abs(x - x0) + 1;
	}

	public int getDataHeight() {
		return Math.abs(y - y0) + 1;
	}

	public int getX() {
		return x;
	}

	public void shiftLeft() {
		isRegion = true;
		x--;
		if (x <= 0)
//			x += layerList.size();
			x += layerList.getColumnCount();
		notifyToCoordinateListeners();
	}

	public void shiftRight() {
		isRegion = true;
		x++;
//		if (x > layerList.size())
		if (x > layerList.getColumnCount())
//			x -= layerList.size();
			x -= layerList.getColumnCount();
		notifyToCoordinateListeners();
	}

	public void left() {
		x--;
		if (x <= 0)
//			x += layerList.size();
			x += layerList.getColumnCount();
		cancelRegion();
		notifyToCoordinateListeners();
	}

	public void right() {
		x++;
//		if (x > layerList.size())
		if (x > layerList.getColumnCount())
//			x -= layerList.size();
			x -= layerList.getColumnCount();
		cancelRegion();
		notifyToCoordinateListeners();
	}

	public void down() {
		y++;
//		if (y > layerList.size())
		if (y > layerList.getRowCount())
//			y -= layerList.size();
			y -= layerList.getRowCount();
		cancelRegion();
		notifyToCoordinateListeners();
	}

	public void up() {
		y--;
		if (y <= 0)
//			y += layerList.size();
			y += layerList.getRowCount();
		cancelRegion();
		notifyToCoordinateListeners();
	}

	public int getY() {
		return y;
	}

	public void shiftUp() {
		isRegion = true;
		y--;
		if (y <= 0)
//			y += layerList.size();
			y += layerList.getRowCount();
		notifyToCoordinateListeners();
	}

	public void shiftDown() {
		isRegion = true;
		y++;
//		if (y > layerList.size())
		if (y > layerList.getRowCount())
//			y -= layerList.size();
			y -= layerList.getRowCount();
		notifyToCoordinateListeners();
	}

	public void setX0(int x) {
		this.x0 = x;
	}

	public int getY0() {
		return y0;
	}

	public boolean isRegion() {
		return isRegion;
	}

	public void setY0(int y0) {
		this.y0 = y0;
	}

	public int getX0() {
		return x0;
	}

	public void setRegion(boolean b) {
		isRegion = b;
	}

	public void setX(int x) {
		this.x = x;
		notifyToCoordinateListeners();
	}

	public void setY(int y) {
		this.y = y;
		notifyToCoordinateListeners();
	}
/*
	public int size() {
		return layerList.size();
	}
*/
//	public int getWidth() {
	//	return lay
//	}
	public void notifyToCoordinateListeners() {
		for (CoordinateListener listener : listeners) {
			listener.coordinateChanged(new CoordinateChangeEvent(this));
		}
	}

	public void addCoordinateListener(CoordinateListener listener) {
		listeners.add(listener);
	}

	public int getYMin() {
		return Math.min(y, y0);
	}

	public int getYMax() {
		return Math.max(y, y0);
	}

	public int getXMin() {
		return Math.min(x, x0);
	}

	public int getXMax() {
		return Math.max(x, x0);
	}

}
